package clientpart;
import java.sql.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.util.Date;
/**
 * Implementa las operaciones que se pueden hacer en la base de datos por parte del cliente.
 * @author andrux
 */
public class OperacionesClienteBD {
   private String bd = "rhc";
   private String login = "root";
   private String passBD = "";
   private String hostname = "localhost";
   private String url="jdbc:mysql://"+hostname+"/"+bd;
   private String[][] diaOp;
   private ArrayList diaLista;
   private final int INTERVALOS=10;
   /**
    * Fija los valores por omisión:<br><br>
    * Nombre Base de datos: "rhc"<br>
    * Nombre de usuario de BD: "root"<br>
    * Password de BD: ""<br>
    * Hostname: "localhost"<br>
    */
   public OperacionesClienteBD(){
   }

   /**
    * Establece los parametros para la conexion.
    * @param login Nombre de usuario de la Base de Datos.
    * @param passBD Contraseña del usuario de la base datos.
    * @param hostname direccion IP, o nombre del host donde aloja la Base datos.
    * @param bd Nombre de la base de datos MySQL.
    */
    public OperacionesClienteBD(String login, String passBD, String hostname, String bd){
        this.login=login;
        this.passBD=passBD;
        this.hostname=hostname;
        this.bd=bd;
        url = "jdbc:mysql://"+hostname+"/"+bd;
     }

    /**
     * Almacena el horario del usuario administrador
     * @param user nombre de usuario de la administración
     * @param dia string inicializado de 7x3 donde se almacenará la información de los días.
     *
     * @return True si el String en el string ha logrado ser almacenado la información de los días.
     */
    public boolean verHorario(String user, String[][] dia){
        String[] diaDeLaSemana={"Lunes","Martes","Miercoles", "Jueves", "Viernes",
                                "Sabado", "Domingo"};
         try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
         for(int i=0;i<7;i++){

            ResultSet res  = stmt.executeQuery("SELECT horainicio, horatermino FROM "+diaDeLaSemana[i]+" " +
                                                "WHERE user='"+user+"'");
             
            while(res.next()){
                dia[i][0]=diaDeLaSemana[i];
                dia[i][1]=res.getString("horainicio");
                dia[i][2]=res.getString("horatermino");
            }
            diaOp=dia;
            res.close();
         }

            stmt.close();
            conn.close();
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
    return false;
    }

    /**
     * Obtiene el mensaje del Administrador que será mostrado al comienzo.
     * @param user nombre de usuario del administrador
     * @return el mensaje
     */
    public String getMensaje(String user){
        String mensaje=null;
         try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            ResultSet res  = stmt.executeQuery("SELECT mensaje FROM admin " +
                                                "WHERE user='"+user+"'");

            while(res.next()){
                mensaje=res.getString("mensaje");
            }

            res.close();
            stmt.close();
            conn.close();
            return mensaje;
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
    return mensaje;
    }

    /**
     * Establece las proximas fechas en las que se pueden hacer reservas
     * @return fechas en el formato yyyy-MM-dd
     */
    public String[] proximasFechas(){
        int j=0,count=0,k=0;
        diaLista=new ArrayList();
        String[] proximasFechas=null;
        Calendar calendario =new GregorianCalendar();
        Date miDate=calendario.getTime();

        SimpleDateFormat formatoFecha = new SimpleDateFormat("yyyy-MM-dd");
        formatoFecha.format(miDate);

        while(j<7){
           // if(diaOp[j][1]!=null && diaOp[j][2]!=null ||
                  if(   diaOp[j][1].compareTo("00:00:00")!=0 &&
                    diaOp[j][2].compareTo("00:00:00")!=0){
                
                if(diaOp[j][0].compareTo("Lunes")==0)
                    diaLista.add(2);
                else if(diaOp[j][0].compareTo("Martes")==0)
                            diaLista.add(3);
                else if(diaOp[j][0].compareTo("Miercoles")==0)
                            diaLista.add(4);
                else if(diaOp[j][0].compareTo("Jueves")==0)
                            diaLista.add(5);
                else if(diaOp[j][0].compareTo("Viernes")==0)
                            diaLista.add(6);
                else if(diaOp[j][0].compareTo("Sabado")==0)
                            diaLista.add(7);
                else if(diaOp[j][0].compareTo("Domingo")==0)
                            diaLista.add(1);
                count++; // cuenta en cuantos días hay horas de consulta
            }
            j++;
        }
        proximasFechas=new String[count*2];//cuantos días con consultas hay en 2 semanas

        //Fechas disponibles los próximos 14 días
        for(j=0;j<14;j++){
            //Si ese día de la semana está contenido en la lista.
            if(diaLista.contains(calendario.get(calendario.DAY_OF_WEEK))){
                proximasFechas[k]=formatoFecha.format(miDate);
                k++;
            }
                calendario.add(Calendar.DATE,1);
                miDate=calendario.getTime();
        }
        
        return proximasFechas;
    }

    /**
     * Establece que horas de consultas están disponibles para un un fecha y un usuario
     * determinados.
     * @param fecha Día que será consultado en el formato aaaa-mm-dd
     * @param user nombre de usuario.
     * @return horas disponibles
     */
    public ArrayList horasConsultasDisponibles(String fecha, String user){
        int nroDia;
        //parsea la fecha introducida..ej 2000-12-30
        int indx=fecha.indexOf("-",0);// indx=1
        String dS=fecha.substring(0,indx);// d="1"
        int indx2=fecha.indexOf("-",indx+1);//indx2=3
        String mS=fecha.substring(indx+1,indx2);//m="2"
        int len=fecha.length();//len=8
        String aS=fecha.substring(indx2+1,len);//a="2002"

        //convierte de Strings a enteros
        int a=Integer.valueOf(dS).intValue();
        int m=Integer.valueOf(mS).intValue();
        int d=Integer.valueOf(aS).intValue();
        ArrayList horaOcupada=new ArrayList();
        String horaInicio=null,horaTermino=null;
        Calendar calendario =new GregorianCalendar(a,m-1,d);//m-1 porque meses van de 0 a 11
        nroDia=calendario.get(calendario.DAY_OF_WEEK);

        try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            ResultSet res1  = stmt.executeQuery("SELECT hora FROM reservas WHERE user='"+user+"' " +
                    "AND fecha='"+a+":"+m+":"+d+"'");

            while(res1.next()){
                //Contendrá las horas que ya están reservadas para que no estén
                //disponibles al usuario que pide una nueva hora de consulta
                horaOcupada.add(res1.getString("hora"));
            }
            ResultSet res2  = stmt.executeQuery("SELECT horainicio,horatermino FROM "+convertNum2Dia(nroDia)+" WHERE user='"+user+"' ");

            while(res2.next()){
                horaInicio=res2.getString("horainicio");
                horaTermino=res2.getString("horatermino");
            }

            res1.close();
            res2.close();
            stmt.close();
            conn.close();
            
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación horaConsultasDisponibles error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación horaConsultasDisponibles error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación horaConsultasDisponibles error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación horaConsultasDisponibles error: "+ex);
       }

    int total=1440/INTERVALOS;//un día tiene 1440 minutos
    int i=0;
    //String[] totalIntervalos = new String[total];
    ArrayList totalIntervalos= new ArrayList();
    Calendar calendarioH =new java.util.GregorianCalendar(1986, Calendar.AUGUST, 17);
    String horaActual;
    System.out.println("HoraActual: "+horaOcupada);
    do{
      java.util.Date myDate=calendarioH.getTime();
      SimpleDateFormat formatHora = new SimpleDateFormat("HH:mm:ss");
      horaActual =formatHora.format(myDate);

      //Si hora hora actual es mayor que la hora de inicio, es menor que la hora de
      //término y no está reservada anteriormente la agrega a la lista.
      if(compararHoras(horaInicio,horaActual) && !compararHoras(horaTermino,horaActual)
              && !horaOcupada.contains(horaActual)){
        //totalIntervalos[i]= horaActual;
          totalIntervalos.add(horaActual);
      }
      calendarioH.add(Calendar.MINUTE, INTERVALOS);
      i++;
    }while(i<total);
    return totalIntervalos;
   }

    /**
     * Determina el nombre del dia de acuerdo a los valores entrgados.
     * @param numDia Numero del día de 1 a 7 (1=Domingo)
     * @return Nombre del dia al que corresponde el número
     */
    private String convertNum2Dia(int numDia){
        switch(numDia){
            case 2: return "lunes";
            case 3: return "martes";
            case 4: return "miercoles";
            case 5: return "jueves";
            case 6: return "viernes";
            case 7: return "sabado";
            case 1: return "domingo";
        }
        return null;
    }

    /**
     * Compara horas.
     * @param s1 hora en formato hh:mm:ss
     * @param s2 hora en formato hh:mm:ss
     * @return True si s1 es menor que s2. False en caso contrario.
     */
    public boolean compararHoras(String s1, String s2){
         //parsea la fecha introducida..ej 2000-12-30
        int indx=s1.indexOf(":",0);// indx=1
        String h1S=s1.substring(0,indx);// d="1"
        int indx2=s1.indexOf(":",indx+1);//indx2=3
        String m1S=s1.substring(indx+1,indx2);//m="2"

        //convierte de Strings a enteros
        int h1=Integer.valueOf(h1S).intValue();
        int m1=Integer.valueOf(m1S).intValue();

        indx=s2.indexOf(":",0);// indx=1
        String h2S=s2.substring(0,indx);// d="1"
        indx2=s2.indexOf(":",indx+1);//indx2=3
        String m2S=s2.substring(indx+1,indx2);//m="2"
       
        int h2=Integer.valueOf(h2S).intValue();
        int m2=Integer.valueOf(m2S).intValue();
        
        if(h1<h2)
            return true;
        if(h1==h2 && m1<=m2)
            return true;
        return false;
    }

    /**
     * Ingresa una nueva reserva a la base de datos.
     * @param fecha
     * @param hora
     * @param nombre
     * @param rol
     * @param email
     * @param motivo
     * @param user
     * @return
     */
    public boolean insertarReserva(String fecha, String hora, String nombre, String rol,
            String email, String motivo, String user){
        try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            int res  = stmt.executeUpdate("INSERT INTO reservas (fecha,hora,nombre,rol,email,motivo,user) " +
                    "VALUES ('"+fecha+"','"+hora+"','"+nombre+"','"+rol+"','"+email+"','"+motivo+"','"+user+"')");
            if(res!=0)
                System.out.println("INSERT OK");

            stmt.close();
            conn.close();
            return true;
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación INSERT error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación INSERT error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación INSERT error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación INSERT error: "+ex);
       }
    return false;
   }

}
